VERSION 5.00
Begin VB.Form frmDragDropFiles 
   AutoRedraw      =   -1  'True
   Caption         =   "DragDrop Files Demo"
   ClientHeight    =   3480
   ClientLeft      =   1575
   ClientTop       =   1530
   ClientWidth     =   3120
   LinkTopic       =   "Form1"
   ScaleHeight     =   232
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   208
   Begin VB.CheckBox Check1 
      Caption         =   "Use OLE DragDrop"
      Height          =   195
      Left            =   210
      TabIndex        =   2
      Top             =   3090
      Width           =   2175
   End
   Begin VB.PictureBox Picture1 
      AutoRedraw      =   -1  'True
      Height          =   2130
      Left            =   180
      ScaleHeight     =   2070
      ScaleWidth      =   2715
      TabIndex        =   0
      Top             =   750
      Width           =   2775
   End
   Begin VB.Label Label1 
      Caption         =   "Open Windows Explorer and drag any valid picture file onto the picturebox."
      Height          =   495
      Left            =   240
      TabIndex        =   1
      Top             =   135
      Width           =   2715
   End
End
Attribute VB_Name = "frmDragDropFiles"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
  Option Explicit
  ' demo project showing how to impliment Drag & Drop in a VB app
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.
  
  ' IMPORTANT NOTE:  if you don't have the DbgwProc.dll Debug Object from
  ' the MS site, you should get it and use it with this project while in
  ' the IDE.  before compiling an app that uses the debug object, you should
  ' set the DEBUGWINDOWPROC conditional compiliation variable equal to zero.
  ' ALSO NOTE: when using subclassing techniques, it is improtant to allow
  ' the normal shutdown and clean up in the project to take place.  in other
  ' words, DO NOT USE THE "STOP" BUTTON IN THE VB IDE.
  
  ' dimension our subclass object
  Private WithEvents m_oSubclass As cSubclass
  
  ' drag and drop files API calls for the API version of drag & drop
  Private Declare Sub DragAcceptFiles Lib "shell32.dll" (ByVal hWnd&, ByVal fAccept&)
                               
  Private Declare Function DragQueryFile Lib "shell32.dll" Alias "DragQueryFileA" (ByVal hDrop&, ByVal iFile&, _
                                                                                  ByVal lpszFile$, ByVal cch&) As Long
  Private Declare Sub DragFinish Lib "shell32.dll" (ByVal hDrop&)
  
Private Sub Form_Load()
  
  ' create an instance of our subclass object and initiate the subclass of our picturebox
  Set m_oSubclass = New cSubclass
  m_oSubclass.SubClass Picture1.hWnd
  
  ' register picture1 as a window that accepts dragdrop files for the API style drag drop
  Call DragAcceptFiles(Picture1.hWnd, 1&)

End Sub

Private Sub Form_Unload(Cancel As Integer)
  ' destroy the subclass object
  Set m_oSubclass = Nothing
End Sub

Private Sub m_oSubclass_FileDropped(ByVal hDrop As Long)

  Dim sFileName$, nCharsCopied&
  
  ' make some space for the file name
  sFileName = String$(MAX_PATH, vbNullChar)
  ' pass the file handle (hDrop), the index of the file if more than 1 was passed (we
  ' still use index zero since we only care about the first file in the list), the variable
  ' that will accept the file name and the amount of space that that variable is dimentioned for.
  nCharsCopied = DragQueryFile(hDrop, 0&, sFileName, MAX_PATH)

  ' clean up after ourselves bu closing the file handle
  DragFinish hDrop
  
  ' if there were chars copied, get the file name and try to load it into the picturbox
  If nCharsCopied Then
    sFileName = Left$(sFileName, nCharsCopied)
  
    ' incase it's not a valid picture display the error message
    On Error GoTo invalidPicture
    
    With Picture1
      ' load the picture
      .Picture = LoadPicture(sFileName)
      
      ' save the file name in case we need to drag it from the picturebox later
      .Tag = sFileName
    End With
  End If
  
  Exit Sub
  
invalidPicture:

  ' display the invalid file format message
  DisplayPicture1Message
  
End Sub

Private Sub Picture1_OLEDragDrop(Data As DataObject, Effect As Long, Button As Integer, Shift As Integer, X As Single, Y As Single)

  ' check the format of the data that is being dropped
  If Data.GetFormat(vbCFFiles) = True Then
  
    Dim sFileName$
    
    ' grab the first file name in the collection of file names
    sFileName = Data.Files(1)
  
    ' incase it's not a valid picture display the error message
    On Error GoTo invalidPicture
    
    ' try to load a picture
    With Picture1
      .Picture = LoadPicture(sFileName)
      .Tag = sFileName
    End With
  End If
  
  Exit Sub
  
invalidPicture:

  ' display the invalid file format message
  DisplayPicture1Message
  
End Sub
                                                              
Private Sub Check1_Click()
  ' toggle between OLE dragdrop or Subclassed dragdrop mode.
  With Picture1
    .OLEDropMode = Check1.Value
    .OLEDragMode = Check1.Value
  End With
End Sub

Private Sub DisplayPicture1Message()

  ' clear any picture out of the control
  Picture1.Picture = LoadPicture()
  
  Const Msg As String = "Invalid Picture Format!"
  
  ' print the error message on the picturebox
  Picture1.CurrentX = (Picture1.ScaleWidth \ 2) - (Picture1.TextWidth(Msg) \ 2)
  Picture1.CurrentY = (Picture1.ScaleHeight \ 2) - (Picture1.TextHeight(Msg) \ 2)
  Picture1.Print Msg

End Sub

Private Sub Picture1_OLEDragOver(Data As DataObject, Effect As Long, Button As Integer, Shift As Integer, X As Single, Y As Single, State As Integer)
  
  ' check the data to see if it is what we will allow.  if not so "no drop"
  If Data.GetFormat(vbCFFiles) Then
    Effect = vbDropEffectCopy And Effect
  Else
    Effect = vbDropEffectNone
  End If

End Sub

Private Sub Picture1_OLEStartDrag(Data As DataObject, AllowedEffects As Long)
  ' this evant allows us to implement dragging *from* our picturebox to other drag targets in the system
  
  ' make sure there is a file name in the tag property (you would probaby want to use a "file
  ' exists" routine here but for this demo we are doing it simply)
  If Picture1.Tag <> "" Then
    On Error GoTo EH
    
    ' set the drop effect to create a copy of the file wherever it get's dropped
    AllowedEffects = vbDropEffectCopy
    
    With Data
      ' clear the contents of the data object
      .Clear
      ' tell the data object we will be dragging files
      .SetData , vbCFFiles
      ' add the file name that we storred in the tag eairler
      .Files.Add Picture1.Tag
    End With
  End If
  
ExitNow:
  Exit Sub
  
EH:
  Err.Clear
  MsgBox "Unable to initiate drag."
  Resume ExitNow
  
End Sub

Private Sub Picture1_OLECompleteDrag(Effect As Long)
  
  ' determine what we will do when the file is dropped depending on the effect param
  Select Case Effect
    Case vbDropEffectNone
      ' do nothing
    Case vbDropEffectCopy, vbDropEffectMove
      ' if the file was dropped successfully, remove the picture from the picturebox
      Set Picture1.Picture = Nothing
    
  End Select
End Sub

